--[[
	BagnonCore
		Core bag functionality
		I think this file looks fairly unorganized.
		
	BagSlots:
		-1: Bank
		0:  Main Bag
		1, 2, 3, 4:   Extra Bags
		5, 6, 7, 8:  Bank Bags
		
	TODO
		Ammo bag detection
--]]

--Local constants
local DEFAULT_COLS = 8;
local DEFAULT_SPACING = 2;

local NORMAL_BG = "Interface\\DialogFrame\\UI-DialogBox-Background";
local OPAQUE_BG = "Interface\\CharacterFrame\\UI-Party-Background";

--[[
	Load settings for the frame
--]]

function BagnonCore_LoadSettings(frame, maxSize, addBags, frameType)
	--another trick taken from OneBag, allows for the nice escape functionality
	tinsert(UISpecialFrames, frame:GetName());
	
	if(not BagnonSets[frame:GetName()] ) then
		BagnonSets[frame:GetName()] = {};
	end
	
	if(BagnonSets[frame:GetName()].alpha) then
		frame:SetAlpha(BagnonSets[frame:GetName()].alpha);
	end
	
	if(BagnonSets[frame:GetName()].bg) then
		BagnonCore_SetBackground(frame, OPAQUE_BG);
	end
	
	if(BagnonForever) then
		BagnonForever_LoadSettings(frame, frameType);
	end
	
	getglobal(frame:GetName() .. "Title"):SetText(UnitName("player") .. "'s " .. frameType);
	frame:RegisterForClicks("LeftButtonDown", "LeftButtonUp", "RightButtonUp");
	frame.MAX_SLOTS = maxSize; --a semi constant
	frame.addBags = addBags; --function used to add all the bags to the frame when generating it
	
	--reposition actually handles rescaling
	BagnonCore_Reposition(frame);
	--This would probably be better named layout
	BagnonCore_Build(frame, BagnonSets[frame:GetName()].cols, BagnonSets[frame:GetName()].space);
	BagnonCore_GenerateFrame(frame);
end

--[[
	Generate the frame (add all items, resize)
--]]

function BagnonCore_GenerateFrame(frame)
	frame.size = nil;
	frame.addBags();
	BagnonCore_TrimToSize(frame);
	
	frame:Show();
end

--[[ 
	Update all the slots in <frame>
--]]

function BagnonCore_Update(frame)
	if(not frame.size) then
		return;
	end

	local frameName = frame:GetName();
	local slot;
	
	--update only the slots the player can use
	for slot = 1, frame.size, 1 do
		local item = getglobal( frameName .. "Item" .. slot );
		
		BagnonCore_UpdateItemInfo(item);
	end
end

--[[
	Highlight all the slots of <bag>
--]]

function BagnonCore_HighlightSlots(frame, bagID)
	if(not frame.size) then
		return;
	end

	local frameName = frame:GetName();
	local slot;

	--update only the slots the player can use
	for slot = 1, frame.size, 1 do
		local item = getglobal( frameName .. "Item" .. slot );

		if(item.bag == bagID) then
			item:LockHighlight();
		end
	end
end

function BagnonCore_UnhighlightAll(frame)
	if(not frame.size) then
		return;
	end

	local frameName = frame:GetName();
	local slot;

	--update only the slots the player can use
	for slot = 1, frame.size, 1 do
		getglobal( frameName .. "Item" .. slot ):UnlockHighlight();
	end
end

--[[ 
	Add all the slots of <bag> to the frame, and increase the total size of frame to include the bags
--]]

function BagnonCore_AddBag(frame, bagID)

	local frameName = frame:GetName();
	local slot, index; --slot refers to the itemslot in <frame>, index refers to the item slot in <bag>
	
	if(not frame.size) then
		frame.size = 0;
		slot = 0;
	else
		slot = frame.size;
	end
	
	--update used slots
	local size = GetContainerNumSlots(bagID);
	
	for index = 1, size, 1 do
		slot = slot + 1;

		local item = getglobal( frameName .. "Item".. slot);
		
		if(not item) then
			frame.size = slot - 1;
			return;
		end
		
		item:SetID(index);
		item.bag = bagID;

		BagnonCore_UpdateItemInfo(item);
		
		item:Show();
	end
	
	frame.size = slot;
end

--[[
	Update the texture, lock status, and other information about an item
--]]

function BagnonCore_UpdateItemInfo(item)
	local texture, itemCount, locked, quality, readable = GetContainerItemInfo(item.bag, item:GetID());
	
	BagnonItem_UpdateBorder(item);
	SetItemButtonTexture(item, texture);
	SetItemButtonCount(item, itemCount);
	SetItemButtonDesaturated(item, locked, 0.5, 0.5, 0.5);
	
	if ( texture ) then
		BagnonCore_UpdateCooldown(item.bag, item);
		item.hasItem = 1;
	else
		getglobal(item:GetName() .. "Cooldown"):Hide();
		item.hasItem = nil;
	end

	item.readable = readable;
end

--[[ 
	Update cooldown
		Not using the default one because I can add cooldown count later
--]]

function BagnonCore_UpdateCooldown(container, button)
	local cooldown = getglobal(button:GetName().."Cooldown");
	local start, duration, enable = GetContainerItemCooldown(container, button:GetID());
	
	CooldownFrame_SetTimer(cooldown, start, duration, enable);
	
	if ( duration > 0 and enable == 0 ) then
		SetItemButtonTextureVertexColor(button, 0.4, 0.4, 0.4);
	end
end

--[[ 
	Hide unusable slots and resize the frame
--]]

function BagnonCore_TrimToSize(frame)
	if(not frame.size) then
		return;
	end
	
	local frameName = frame:GetName();
	local slot;
	
	--hide unused slots
	for slot = frame.size + 1, frame.MAX_SLOTS, 1 do
		getglobal( frameName .. "Item".. slot ):Hide();
	end
	
	--set the frame's width
	--24 is the estimated border width of the frame, 37 is about the width of an item slot
	if(frame.size < frame.cols) then
		frame:SetWidth( ( 37 + frame.space ) * frame.size + 24 - frame.space );
	else
		frame:SetWidth( ( 37 + frame.space ) * frame.cols + 24 - frame.space );
	end
	
	--set the frame's height, adjusted to fit a bag frame if its shown/there is one
	
	local bagFrame = getglobal(frame:GetName() .. "Bags");	
	local height = ( 37 + frame.space ) * math.ceil( frame.size / frame.cols )  + 64 - frame.space;
	
	if(bagFrame and bagFrame:IsShown() ) then
		frame:SetHeight(height + bagFrame:GetHeight());
		
		if(frame:GetWidth() < bagFrame:GetWidth()) then
			frame:SetWidth(bagFrame:GetWidth());
		end
	else
		frame:SetHeight(height);
	end
	
	Infield_Reposition(frame, UIParent);
	BagnonCore_SavePosition(frame);
end

--[[
	Reposition all the item slots in <frame> to be in <cols> columns and spaced <space>
--]]

function BagnonCore_Build(frame, cols, space)
	if(frame.size and frame.size == 0) then
		return;
	end
	
	local frameName = frame:GetName();
	
	if(not cols) then
		cols = DEFAULT_COLS;
	end
	
	if(not space) then
		space = DEFAULT_SPACING;
	end
	
	if(cols == DEFAULT_COLS) then
		BagnonSets[frameName].cols = nil
	else
		BagnonSets[frameName].cols = cols
	end
	
	if(space == DEFAULT_SPACING) then
		BagnonSets[frameName].space = nil;
	else
		BagnonSets[frameName].space = space;
	end
	
	local rows = math.ceil( frame.MAX_SLOTS / cols );
	local index = 1;
	local button;
	
	--resize the frame
	frame.cols = cols;
	frame.space = space;
	
	
	button = getglobal(frameName .. "Item1");
	button:ClearAllPoints();
	button:SetPoint("TOPLEFT", button:GetParent(), "TOPLEFT", 12, -32);
	
	for i = 1, rows, 1 do 
	
		for j = 1, cols, 1 do
		
			index = index + 1;
			
			button = getglobal(frameName .. "Item" .. index);
			
			if(not button) then
				break;
			end
			
			button:ClearAllPoints();
			button:SetPoint("LEFT", frameName .. "Item" .. index - 1, "RIGHT", space, 0);
		end
		
		button = getglobal(frameName .. "Item" .. index);
		
		if(not button) then
			break;
		end
		
		button:ClearAllPoints();
		button:SetPoint("TOP", frameName .. "Item" .. index - cols, "BOTTOM", 0, -space);
	end
	
	BagnonCore_TrimToSize(frame);
end

--change <frame>'s transparency
function BagnonCore_SetAlpha(frame, alpha)
	if(alpha ~= 1) then
		BagnonSets[frame:GetName()].alpha = alpha;
	else
		BagnonSets[frame:GetName()].alpha = nil
	end
	frame:SetAlpha(alpha);
end

--Set and scale <frame>
function BagnonCore_SetScale(frame, scale)
	BagnonSets[frame:GetName()].scale = scale;
	
	Infield_Scale(frame, scale);
	BagnonCore_SavePosition(frame);
end

function BagnonCore_SetBackground(frame, background)
	if(background == NORMAL_BG) then
		BagnonSets[frame:GetName()].bg = nil;
	else
		BagnonSets[frame:GetName()].bg = 1;
	end
	frame:SetBackdrop({bgFile=background, edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=false, tileSize=16, edgeSize=16, insets={left=4, right=4, top=4, bottom=4}})
end

--[[ 
	Positioning Functions 
--]]

function BagnonCore_StartMoving(frame)
	 if(not BagnonSets[frame:GetName()].locked) then
		frame:StartMoving();
	end
end

function BagnonCore_StopMoving(frame)
	frame:StopMovingOrSizing();
	Infield_Reposition(frame, UIParent);
	BagnonCore_SavePosition(frame);
end

--Place the frame at the last place it was at.  
--This is used when the frame first loads because the game currently does not remember the last position of a frame that's dynamically loaded
function BagnonCore_Reposition(frame)
	local frameName = frame:GetName();
	
	if(not BagnonSets[frameName] or not BagnonSets[frameName].top) then
		return;
	end
	
	local ratio;
	--this step is to take care of a potential flaw if the UI scale changes between loadings of the frame
	if(BagnonSets[frameName].parentScale ) then
		ratio = BagnonSets[frameName].parentScale / frame:GetParent():GetScale();
	else
		ratio = 1;
	end
	
	frame:ClearAllPoints();
	frame:SetScale(BagnonSets[frameName].scale);
	frame:SetPoint("TOPLEFT", frame:GetParent(), "BOTTOMLEFT", BagnonSets[frameName].left * ratio, BagnonSets[frameName].top * ratio);
end

--Save the frame's current position.  Needed because frames don't remember their positions if dynamically loaded.
function BagnonCore_SavePosition(frame)
	local frameName = frame:GetName();
	
	if(not BagnonSets[frameName] ) then
		BagnonSets[frameName] = {};
	end
	
	BagnonSets[frameName].top = frame:GetTop();
	BagnonSets[frameName].left = frame:GetLeft();
	BagnonSets[frameName].scale = frame:GetScale();
	BagnonSets[frameName].parentScale = frame:GetParent():GetScale();
end

function BagnonCore_ToggleLock(frame, checked)
	local frameName = frame:GetName();
	
	if(checked) then
		BagnonSets[frameName].locked = 1;
	else
		BagnonSets[frameName].locked = nil;
	end
end

--[[
	Open/Close/Toggle <frame>
--]]

function BagnonCore_Toggle(frameName)
	local frame = getglobal(frameName);
	if( frame ) then
		if ( frame:IsVisible() ) then
			BagnonCore_Close(frameName);
		else
			BagnonCore_Open(frameName);
		end
	else
		LoadAddOn(frameName);
	end
end

function BagnonCore_Open(frameName)
	local frame = getglobal(frameName);
	if( frame ) then
		BagnonCore_GenerateFrame(frame);
	else
		LoadAddOn(frameName);
	end
end

function BagnonCore_Close(frameName)
	local frame = getglobal(frameName);
	if ( frame ) then
		frame:Hide();
	end
end

--[[
	Rightclick Menu Stuff
--]]

--this function is there so that it can be overriden
function BagnonCore_OnDoubleClick(frame)
	return;
end

function BagnonCore_OnClick(frame, mouseButton)
	if(mouseButton == "RightButton") then
		BagnonCore_ShowMenu(frame);
	end
end

function BagnonCore_ShowMenu(frame)
	BagnonMenu.onShow = 1;
	BagnonMenu.frame = frame;
	BagnonMenuText:SetText(frame:GetName() .. " Settings");
	
	--Set values
	BagnonMenuLocked:SetChecked(BagnonSets[frame:GetName()].locked);
	BagnonMenuOpaque:SetChecked(BagnonSets[frame:GetName()].bg);
	BagnonMenuColumns:SetValue(frame.cols);
	BagnonMenuSpacing:SetValue(frame.space);
	BagnonMenuScale:SetValue(frame:GetScale() * 100);
	BagnonMenuOpacity:SetValue(frame:GetAlpha() * 100);
	
	--a nifty thing I saw in meta map adapted for my usage
	--places the options menu at the cursor's position
	local x, y = GetCursorPosition();
	x = x / UIParent:GetScale();
	y = y / UIParent:GetScale();
	
	BagnonMenu:ClearAllPoints();
	Infield_Place(BagnonMenu, "TOPLEFT", UIParent, "BOTTOMLEFT", x, y);
	
	BagnonMenu:Show();
	BagnonMenu.onShow = nil;
end

function BagnonCore_ToggleBG(frame, checked)
	if( checked ) then
		BagnonCore_SetBackground(frame, OPAQUE_BG);
	else
		BagnonCore_SetBackground(frame, NORMAL_BG);
	end
end

--[[
	Other Stuff
--]]

function BagnonCore_IsAmmoBag(bagID)
	--bank and the main bag are never ammo bags
	if(bagID == 0 or bagID == -1) then
		return nil;
	end
	
	
	local link = GetInventoryItemLink("player", ContainerIDToInventoryID(bagID) );
	
	
	if(link) then
		local _, _, id = string.find(link, "item:(%d+)");
		local _, _, _, _, _, subType = GetItemInfo(id);

		return (subType ~= "Bag" and string ~= "Miscellaneous");
	end
	
	return nil;
end

function BagnonCore_IsInventoryBag(bagID)
	return (bagID >= 0 and bagID < 5);
end

function BagnonCore_IsBankBag(bagID)
	return (bagID == -1 or (bagID >= 5 and bagID < 11) );
end