--[[
	BagnonDB
		Functions for storing and accessing player inventory data
		This is intended to be completely seperate from Bagnon_Forever
		
	BagnonData has this format, which was taken from KC_Items
		Realm
			Character
				Bag
					Size
					ItemSlot
						link,icon,[count]
				Money
--]]

--local constants
local ICON_PATH = "Interface\\Icons\\";

local LINK_COLOR = {
	[0] = "9d9d9d", --grey
	[1] = "ffffff", --common
	[2] = "1eff00", --uncommon
	[3] = "0070dd", -- rare
	[4] = "a335ee", --epic
	[5] = "ff8000", --legendary
	[6] = "e6cc80", --artifact
};

--local globals
local currentPlayer = UnitName("player"); --the current player that's logged on
local realm = GetRealmName(); --what realm we're in


function BagnonDB_OnLoad()
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("BANKFRAME_CLOSED");
	this:RegisterEvent("BANKFRAME_OPENED");
	this:RegisterEvent("PLAYERBANKSLOTS_CHANGED");
	this:RegisterEvent("PLAYER_MONEY");
end

function BagnonDB_OnEvent(event)
	if(event == "BAG_UPDATE") then
		BagnonDB_SaveBagData(arg1);
	elseif(event == "PLAYERBANKSLOTS_CHANGED") then
		BagnonDB_SaveBagData(-1);
	elseif(event == "BANKFRAME_CLOSED") then
		atBank = nil;
	elseif(event == "BANKFRAME_OPENED") then
		atBank = 1;
		BagnonDB_SaveBankData();
	elseif(event == "PLAYER_MONEY" or event == "PLAYER_LOGIN") then
		BagnonDB_SavePlayerMoney();
	elseif(event == "PLAYER_LEAVING_WORLD") then
		this:UnregisterEvent("BAG_UPDATE");
	elseif(event == "PLAYER_ENTERING_WORLD") then
		this:RegisterEvent("BAG_UPDATE");
		BagnonDB_SaveAllData();
	--the multiple overrides are due in part because the game actually reloads the original code whenever Bagnon or Banknon is loaded
	elseif(event == "ADDON_LOADED" and arg1 == "Bagnon_DB") then
		this:UnregisterEvent("ADDON_LOADED");
		BagnonDB_LoadVariables();
	end
end

--initialize data and overrides
function BagnonDB_LoadVariables()
	local realm = GetRealmName(); --what realm we're in
	
	if(not BagnonItems) then
		BagnonItems = {};
		BagnonItems.version = VERSION;
	end
	
	if(not BagnonItems[realm]) then
		BagnonItems[realm] = {};
	end
	
	if(not BagnonItems[realm][UnitName("player")]) then
		BagnonItems[realm][UnitName("player")] = {};
	end
	
	if(BagnonItems.version ~= BAGNON_VERSION) then
		BagnonDB_UpdateVersion();
	end
end

function BagnonDB_UpdateVersion()
	local realm = GetRealmName(); --what realm we're in
		
	BagnonItems = {};
	BagnonItems.version = VERSION;

	BagnonItems[realm] = {};
	BagnonItems[realm][UnitName("player")] = {};
	
	BagnonItems.version = BAGNON_VERSION;
	DEFAULT_CHAT_FRAME:AddMessage("Bagnon Database updated to v" .. BAGNON_VERSION, 1, 1, 0);
end

--[[
	Storage Functions
--]]

function BagnonDB_SaveAllData()
	local i;
	for i = -1, 10, 1 do
		BagnonDB_SaveBagData(i);
	end
	BagnonDB_SavePlayerMoney();
end

function BagnonDB_SaveBankData()
	BagnonDB_SaveBagData(-1);
	local bagID;
	for bagID = 5, 10, 1 do
		BagnonDB_SaveBagData(bagID);
	end
end

--called whenever the bag updates, save all the bag's dat and
function BagnonDB_SaveBagData(bagID)
	--don't save bank data unless you're at the bank
	if( ( bagID == -1 or ( bagID > 4 and bagID <= 10 ) ) and not atBank ) then
		return;
	end
	
	local size = GetContainerNumSlots(bagID);
	
	if(size > 0) then
		--save bag size
		BagnonItems[realm][currentPlayer][bagID] = {};
		BagnonItems[realm][currentPlayer][bagID].s = size;
		
		--save all item info
		for index = 1, size, 1 do
			BagnonDB_SaveItemData(bagID, index);
		end
	else
		BagnonItems[realm][currentPlayer][bagID] = nil;
	end
end

function BagnonDB_SaveItemData(bagID, itemSlot)
	local texture, count = GetContainerItemInfo(bagID, itemSlot);
	local data;
	
	if(texture) then
	
		data = BagnonDB_ToShortLink( GetContainerItemLink(bagID, itemSlot) ) .. ",";
		if(count > 1) then
			data = data .. count;
		end
	end
	
	BagnonItems[realm][currentPlayer][bagID][itemSlot] = data;
end

function BagnonDB_SavePlayerMoney()
	BagnonItems[realm][currentPlayer].g = GetMoney();
end

--[[
	Access  Functions
--]]

--returns <player>'s current money
function BagnonDB_GetMoney(player)
	return BagnonItems[realm][player].g;
end

--returns the size of the bag for <player> in slot <bagID>
function BagnonDB_GetBagSize(player, bagID)
	local data = BagnonItems[realm][player][bagID];
	if(data) then
		return data.s;
	end
	return 0;	
end

--returns the item link, (in the form of item:www:xxx:yyy:zzz),  texture (full path) , and count of <player>'s <item>
function BagnonDB_GetItemData(item)
	local itemData = BagnonItems[realm][item:GetParent().player][item.bag][item:GetID()];
	
	if(itemData) then
		local _, _, shortLink, count = string.find(itemData, "([%w_:]+),([%w_:]*)");
		
		shortLink = "item:" .. shortLink;
		
		if(count == "") then
			count = 1;
		else
			count = tonumber(count);
		end
		
		local _, _, _, _, _, _, _, _, texture = GetItemInfo(shortLink);
		
		return "item:" .. shortLink, texture, count;
	end
	
	return nil;
end

--return the item:x:y:z:a link format for <player>'s <item>
function BagnonDB_GetItemLink(item)
	local itemData = BagnonItems[realm][item:GetParent().player][item.bag][item:GetID()];
	
	if(itemData) then
		local _, _, shortLink = string.find(itemData, "([%w_:]+),([%w_:]*)");
		return "item:" .. shortLink;
	end
	
	return nil;
end

--return <player>'s <item>'s id (  the w part in the link format item:w:x:y:z )
function BagnonDB_GetItemID(item)
	local itemData = BagnonItems[realm][item:GetParent().player][item.bag][item:GetID()];
	
	if(itemData) then
		local _, _, id = string.find(itemData, "(%d+)");
		return tonumber(id);
	end
	
	return nil;
end

--return the full hyperlink of an item
function BagnonDB_GetFullItemLink(item)
	local id = BagnonDB_GetItemID(item);
	
	if(id) then
		local name, link, quality = GetItemInfo( id );
	
		return "|cff" .. LINK_COLOR[quality] .. "|H".. link .. "|h[" .. name .. "]|h|r";
	end
end

--returns the texture of <player>'s <item>
function BagnonDB_GetItemTexture(item)
	local itemData = BagnonItems[realm][item:GetParent().player][item.bag][item:GetID()];
	
	if(itemData) then
		local _, _, shortLink  = string.find(itemData, "([%w_:]+),([%w_:]*)");
		local _, _, _, _, _, _, _, _, texture = GetItemInfo("item:" .. shortLink);
		return texture;
	end
	
	return nil;
end

--returns the size of the item's stack
function BagnonDB_GetItemCount(item)
	local itemData = BagnonItems[realm][item:GetParent().player][item.bag][item:GetID()];
	
	if(itemData) then
		local _, _, shortLink, count = string.find(itemData, "([%w_:]+),([%w_:]*)");
		
		if(count == "") then
			count = 1;
		else
			count = tonumber(count);
		end
		
		return count;
	end
	
	return nil;
end

--takes a full item hyperlink and returns the www:xxx:yyy:zzz form
function BagnonDB_ToShortLink(fullLink)
	if(fullLink) then
		local _, _, w, x, y, z = string.find(fullLink, "(%d+):(%d+):(%d+):(%d+)") ;
		return w .. ":" .. x .. ":" .. y .. ":" .. z;
	end
	return nil;
end

function BagnonDB_AtBank()
	return atBank;
end